package net.wanji.utc.hisense.service.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossLightsPO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.cache.CrossPhaseDirTurnCache;
import net.wanji.utc.hisense.cache.CrossSchemePhaseTimeCountCache;
import net.wanji.utc.hisense.cache.SignalDataCache;
import net.wanji.utc.hisense.common.OperationBaseDom;
import net.wanji.utc.hisense.common.constants.HttpConstants;
import net.wanji.utc.hisense.common.enums.CrossLisghtsLaneDirEnum;
import net.wanji.utc.hisense.common.enums.DataBrainControlModeEnum;
import net.wanji.utc.hisense.pojo.CrossPhaseDirTurnPojo;
import net.wanji.utc.hisense.pojo.convert.HisenseLightStatusPojo;
import net.wanji.utc.hisense.pojo.dto.CrossSchemePhaseCountDownDTO;
import net.wanji.utc.hisense.pojo.dto.PhaseCountDownDTO;
import net.wanji.utc.hisense.service.SignalStatusService;
import net.wanji.utc.hisense.util.OkHttpClientUtil;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:28
 */
@Service
@Slf4j
public class SignalStatusServiceImpl implements SignalStatusService {
    @Value("${hisense.url}")
    private String hisenseUrl;
    @Resource
    private CrossInfoMapper crossInfoMapper;

    @Value("${hisenseLightsStatus.shensiUrl}")
    private  String shensiUrl;
    @Value("${hisenseLightsStatus.baiduUrl}")
    private  String baiduUrl;
    @Value("${hisenseLightsStatus.yiGouUrl}")
    private  String yiGouUrl;

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) throws DocumentException {
        //从缓存中拿路口信息
        CrossInfoPO crossInfoPO = CrossInfoCache.getCrossInfoCache().get(crossId);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_3);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPO.getCode());
        List<SignalStatusLogPO> list = new ArrayList<>();

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.4. 联机状态 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String spot = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Spot");
            //Status：联机状态。0-离线，1-在线。
            String status = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Status");
            SignalStatusLogPO signalStatusLogPO = new SignalStatusLogPO();
            signalStatusLogPO.setCrossId(crossId);
            signalStatusLogPO.setSignalId(crossInfoPO.getCode());
            signalStatusLogPO.setStatus(Integer.valueOf(status));
            if ("1".equals(status)) {
                signalStatusLogPO.setFaultType(0);
            }else {
                signalStatusLogPO.setFaultType(5);
            }
            signalStatusLogPO.setControlType(null);
            signalStatusLogPO.setGmtCreate(null);
            list.add(signalStatusLogPO);

            return list;

        } else {
            //失败
            log.error("请求 5.4. 联机状态 失败");
        }

        return null;
    }

    /**
     * http 形式
     *
     * @param crossId
     * @return
     */
    @Override
    public List<LightsStatusVO> lightStatus(String crossId) {
        List<LightsStatusVO> lightsStatusVOS = new ArrayList<>();
        Map<String, LightsStatusVO> runningStateInfoCache = SignalDataCache.runningStateInfoCache;
        if (!runningStateInfoCache.isEmpty()) {
            LightsStatusVO lightsStatusVO = runningStateInfoCache.get(crossId);
            lightsStatusVOS.add(lightsStatusVO);
            return lightsStatusVOS;
        }
        return lightsStatusVOS;
    }

//    @Override
//    public List<LightsStatusVO> lightStatus(String crossId) {
//        List<LightsStatusVO> lightsStatusVOS = new ArrayList<>();
//        Map<String, RunningLightsStatusPojo> runningStateInfoCache = SignalDataCache.runningStateInfoCacheUdp;
//        if (!runningStateInfoCache.isEmpty()) {
//            RunningLightsStatusPojo runningLightsStatusPojo = runningStateInfoCache.get(crossId);
//            LightsStatusVO lightsStatusVO = new LightsStatusVO();
//            lightsStatusVO.setCrossId(crossId);
//            String runMode = String.valueOf(ControlModelEnum.getWjControlMode(runningLightsStatusPojo.getCCT()));
//            lightsStatusVO.setRunMode(runMode);
//            lightsStatusVO.setCycleLen(runningLightsStatusPojo.getCC());
//            // 相位剩余时长
//            List<RunningLightsStatusPojo.Phase> phaseList = runningLightsStatusPojo.getPHASE();
//            Integer phaseId = 0;
//            Integer phaseCountDown = 0;
//            for (RunningLightsStatusPojo.Phase phase : phaseList) {
//                Integer ls = phase.getLS();
//                if (Objects.equals(ls, 1)) { // 当前相位绿灯
//                    phaseId = phase.getPHN();
//                    phaseCountDown = phase.getPCD();
//                }
//            }
//            lightsStatusVO.setCycleCountDown(phaseCountDown);
//            lightsStatusVO.setPhaseId(String.valueOf(phaseId));
//            lightsStatusVO.setSchemeId(String.valueOf(runningLightsStatusPojo.getPLN()));
//
//            RunningLightsStatusPojo.Channel channel = runningLightsStatusPojo.getCHANNEL();
//            Map<String, Object> dirLampGroupMap = new HashMap<>();
//            List<Integer> scs = channel.getSCS();
//            if (!CollectionUtils.isEmpty(scs)) {
//                for (int i = 0; i < scs.size(); i++) {
//                    Map<Integer, String> turnColorMap = new HashMap<>();
//                    String lightsColor = String.valueOf(scs.get(i));
//                    String wjLightColor = PhaseLightColorEnum.getWjLightColor(lightsColor);
//                    Integer turn = null;
//                    turn = CommonUtils.getTurnByLightsId(i);
//
//                    // 复杂路口处理
////                    Map<String, Map<Integer, Integer>> cacheMap = ComplexCrossCache.getCacheMap();
////                    if (cacheMap.containsKey(crossId)) {
////                        Map<Integer, Integer> innerMap = cacheMap.get(crossId);
////                        if (innerMap.containsKey(i)) {
////                            turn = innerMap.get(i);
////                        }
////                    }
//
//                    turnColorMap.put(turn, wjLightColor);
//                    String dir = String.valueOf(CommonUtils.getDirByLightsId(i));
//                    if (dirLampGroupMap.isEmpty() || Objects.isNull(dirLampGroupMap.get(dir))) {
//                        dirLampGroupMap.put(dir, turnColorMap);
//                    } else {
//                        Map<Integer, String> currentTurnColorMap = (Map<Integer, String>) dirLampGroupMap.get(dir);
//                        currentTurnColorMap.put(turn, wjLightColor);
//                        dirLampGroupMap.put(dir, currentTurnColorMap);
//                    }
//                }
//            }
//            lightsStatusVO.setDirLampGroupMap(dirLampGroupMap);
//            lightsStatusVOS.add(lightsStatusVO);
//        }
//        return lightsStatusVOS;
//    }

    /**
     * @param hisenseLightStatusPojos
     * @throws Exception
     * @description 通过http形式接收海信灯态
     */
    @Override
    public void receiveLightStatus(List<HisenseLightStatusPojo> hisenseLightStatusPojos) throws Exception {
        Date date = new Date();
        Long time = date.getTime();
        ObjectMapper jackson = JacksonUtils.getInstance();
        pushOtherCompany(hisenseLightStatusPojos, jackson, shensiUrl, baiduUrl, yiGouUrl);

        if (!CollectionUtils.isEmpty(hisenseLightStatusPojos)) {
            for (HisenseLightStatusPojo hisenseLightStatusPojo : hisenseLightStatusPojos) {
                String bodyStr = hisenseLightStatusPojo.getBody();
                List<HisenseLightStatusPojo.Body> bodies = jackson.readValue(bodyStr, new TypeReference<List<HisenseLightStatusPojo.Body>>() {});
                if (!CollectionUtils.isEmpty(bodies)) {
                    for (HisenseLightStatusPojo.Body body : bodies) {
                        HisenseLightStatusPojo.ContentBody content = body.getContent();
                        LightsStatusVO lightsStatusVO = new LightsStatusVO();
                        String hiCrossId = content.getCrossId();
                        CrossInfoPO crossInfo = CrossInfoCache.getCrossInfoBySignalCode(hiCrossId);
                        Long signalMachineStamp = content.getSignalMachineStamp();
                        if (Objects.isNull(crossInfo)) {
                            continue;
                        }
                        StaticInfoServiceImpl.planMap.put(crossInfo.getId(), content.getPlanId());
                        String crossId = crossInfo.getId();
                        Long lastSignalMachineStamp = SignalDataCache.runningStatusStampMap.get(crossId);
                        if (Objects.isNull(lastSignalMachineStamp)) {
                            SignalDataCache.runningStatusStampMap.put(crossId, signalMachineStamp);
                        } else {
                            if (lastSignalMachineStamp >= signalMachineStamp) {
                                continue;
                            }
                            SignalDataCache.runningStatusStampMap.put(crossId, signalMachineStamp);
                        }

                        int offsetTime = (int) ((time - signalMachineStamp) / 1000);
                        long currentTimeMillis = System.currentTimeMillis();
                        int localOffset = (int) ((currentTimeMillis - signalMachineStamp) / 1000);
                        log.info("海信灯态延迟时间：{}秒", offsetTime);
                        log.info("海信灯态延迟时间, 本地时间差：{}秒", localOffset);
                        int cycleCountDown = content.getCycleCountDown() - Math.max(offsetTime, 0);
                        int hiControlMode = content.getControlMode();
                        int period = content.getPeriod();
                        int runTime = period - cycleCountDown;
                        int planId = (content.getPlanId() + 2) / 3;
                        lightsStatusVO.setCrossId(crossId);
                        lightsStatusVO.setCode(crossInfo.getCode());
                        lightsStatusVO.setSchemeId(String.valueOf(planId));
                        String runMode = String.valueOf(DataBrainControlModeEnum.getWjControlMode(hiControlMode));
                        lightsStatusVO.setRunMode(runMode);
                        lightsStatusVO.setCycleCountDown(cycleCountDown);
                        lightsStatusVO.setCycleLen(period);
                        lightsStatusVO.setSchemeStartTime(String.valueOf(signalMachineStamp));
                        lightsStatusVO.setTimeStamp(String.valueOf(signalMachineStamp + offsetTime));
                        lightsStatusVO.setPhaseMap(Collections.emptyMap());
                        lightsStatusVO.setControlMode(runMode);
                        lightsStatusVO.setCode(hiCrossId);
                        lightsStatusVO.setManufacturerCode("Hisense");
                        lightsStatusVO.setPhasePlanId(String.valueOf(planId));
                        List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = CrossSchemePhaseTimeCountCache.crossSchemePhaseCountDownList;
                        LightsStatusVO lightsStatusVOCurrent = setLightsStatusVOPhaseNoAndCountDown(lightsStatusVO, crossId, runTime, planId, crossSchemePhaseCountDownList);
                        if (Objects.isNull(lightsStatusVOCurrent)) {
                            log.error("当前海信路口灯态信息异常：{}", lightsStatusVO);
                            continue;
                        }
                        SignalDataCache.runningStateInfoCacheNoLamp.put(crossId, lightsStatusVOCurrent);
                    }
                }
            }
        }
    }

    private static void pushOtherCompany(List<HisenseLightStatusPojo> hisenseLightStatusPojos, ObjectMapper jackson,
                                         String shensiUrl, String baiduUrl, String yiGouUrl) {
        try {
            OkHttpClientUtil.jsonPost(shensiUrl, jackson.writeValueAsString(hisenseLightStatusPojos));
        } catch (Exception e) {
            log.error("海信灯态推送神思失败：{}", e.getMessage());
        }
        try {
            OkHttpClientUtil.jsonPost(baiduUrl, jackson.writeValueAsString(hisenseLightStatusPojos));
        } catch (Exception e) {
            log.error("海信灯态推送百度失败：{}", e.getMessage());
        }
        try {
            HisenseLightStatusPojo hisenseLightStatusPojo = hisenseLightStatusPojos.get(0);
            String bodyStr = hisenseLightStatusPojo.getBody();
            List<HisenseLightStatusPojo.Body> bodies = jackson.readValue(bodyStr, new TypeReference<List<HisenseLightStatusPojo.Body>>() {});
            for (HisenseLightStatusPojo.Body body : bodies) {
                HisenseLightStatusPojo.ContentBody content = body.getContent();
                String crossId = content.getCrossId();
                if (StringUtils.equalsIgnoreCase("255136", crossId)) {
                    OkHttpClientUtil.jsonPost(yiGouUrl, jackson.writeValueAsString(hisenseLightStatusPojos));
                }
            }
        } catch (Exception e) {
            log.error("海信灯态推送易购失败：{}", e);
        }
    }

    /**
     * 通过周期倒计时判断当前相位号，相位倒计时时间
     *
     * @param lightsStatusVO
     * @param crossId
     * @param runTime
     * @param planId
     * @param crossSchemePhaseCountDownList
     */
    private static LightsStatusVO setLightsStatusVOPhaseNoAndCountDown(LightsStatusVO lightsStatusVO, String crossId, int runTime, int planId, List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList) {
        if (!CollectionUtils.isEmpty(crossSchemePhaseCountDownList)) {
            LightsStatusVO lightsStatusVOTemp = new LightsStatusVO();
            for (CrossSchemePhaseCountDownDTO dto : crossSchemePhaseCountDownList) {
                String crossIdCache = dto.getCrossId();
                String schemeNoCache = dto.getSchemeNo();
                if (StringUtils.equalsIgnoreCase(crossIdCache, crossId) && StringUtils.equalsIgnoreCase(schemeNoCache, String.valueOf(planId))) {
                    List<PhaseCountDownDTO> phaseCountDownDTOList = dto.getPhaseCountDownDTOList();
                    int lastPhaseCountTime = 0;
                    int phaseTimeCountDown = 0;
                    String phaseNo = null;
                    for (PhaseCountDownDTO phaseCountDownDTO : phaseCountDownDTOList) {
                        phaseNo = phaseCountDownDTO.getPhaseNo();
                        Integer changePhaseTime = phaseCountDownDTO.getChangePhaseTime();
                        if (lastPhaseCountTime <= runTime && runTime < changePhaseTime) {
                            phaseTimeCountDown = changePhaseTime - runTime;
                            break;
                        }
                        lastPhaseCountTime = changePhaseTime;
                    }
                    BeanUtils.copyProperties(lightsStatusVO, lightsStatusVOTemp);
                    lightsStatusVOTemp.setPhaseId(phaseNo);
                    lightsStatusVOTemp.setCyclePhaseCountDown(phaseTimeCountDown);
                    return lightsStatusVOTemp;
                }
            }
        }
        return null;
    }

    /**
     * @description 通过方向转向获取路口方案的相位号
     */
    private static Integer getCurrentPhagetseNo(String crossId, int planId, Map<Integer, List<Integer>> hisDirTurnMap) {
        Integer phaseNo = null;
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnList = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        if (!CollectionUtils.isEmpty(crossPhaseDirTurnList)) {
            List<CrossPhaseDirTurnPojo> schemeNoList = crossPhaseDirTurnList.stream().filter(po -> Objects.equals(po.getSchemeNo(), planId)).collect(Collectors.toList());
            for (CrossPhaseDirTurnPojo crossPhaseDirTurnPojo : schemeNoList) {
//                Map<Integer, List<Integer>> dirTurnCache = crossPhaseDirTurnPojo.getDirTurnMap();
                Map<Integer, List<CrossLightsPO>> dirTurnMap = crossPhaseDirTurnPojo.getDirTurnMap();
                for (Map.Entry<Integer, List<CrossLightsPO>> dirTurnCacheEntry : dirTurnMap.entrySet()) {
                    Integer dirCache = dirTurnCacheEntry.getKey();
                    for (Map.Entry<Integer, List<Integer>> hisEntry : hisDirTurnMap.entrySet()) {
                        Integer hisDir = hisEntry.getKey();
                        if (Objects.equals(dirCache, CrossLisghtsLaneDirEnum.getLightsDirByLaneDir(hisDir))) {
                            List<CrossLightsPO> turnListCache = dirTurnCacheEntry.getValue();
                            List<Integer> turnList = turnListCache.stream().map(CrossLightsPO::getTurn).sorted().collect(Collectors.toList());
                            if (hisEntry.getValue().stream().allMatch(turnList::contains)) {
                                phaseNo = crossPhaseDirTurnPojo.getPhaseNo();
                            }
                        }
                    }
                }
            }
        }
        return phaseNo;
    }

}
